//   
//  3 (25  2004 )

// Gauss
// version 2.0
// (c) Demidov Vladimir, 2004

//     
//   a[0][0]*x[0] + ... +   a[0][j]*x[j] + ... +   a[0][n-1]*x[n-1] = b[0]
//								...
//   a[i][0]*x[0] + ... +   a[i][j]*x[j] + ... +   a[i][n-1]*x[n-1] = b[i]
//								...
// a[n-1][0]*x[0] + ... + a[n-1][j]*x[j] + ... + a[n-1][n-1]*x[n-1] = b[n-1]

// www.demidovvova.narod.ru
// E-mail: demidovvova@narod.ru

// Microsoft Visual C++ 6.0
// Microsoft Visual C++ 7.0

#include <iostream> //      -
#include <math.h>  
#include <conio.h> //   Linux

using namespace std; //    Turbo C
const double EPS = 1e-10;

void Input_Vector (double *X, int n)
{
	int i;
	for (i=0; i<n; i++)
	{
		cout << "[" << i << "] = "; cin >> X[i];
	}
}

void Output_Vector (double *X, int n)
{
	int i;
	for (i=0; i<n-1; i++)
	{
		cout << X[i] << ", ";
	}
	if (n>0) cout << X[n-1] << endl;
}

void Input_Matrix (double **X, int n)
{
	int i, j;
	for (i=0; i<n; i++)
	{
		for (j=0; j<n; j++)
		{
			cout << "[" << i << ", " << j << "] = "; cin >> X[i][j];
		}
	}
}

void Output_Matrix (double **X, int n)
{
	int i, j;
	for (i=0; i<n; i++)
	{
		for (j=0; j<n-1; j++)
		{
			cout << X[i][j] << "\t";
		}
		//   if (n>0), .. n > i >= 0  =>  n > 0
		cout << X[i][n-1] << "\n";
	}
}

void Output_Matrix_Vector (double **A, double *B, int n)
{
	int i, j;
	for (i=0; i<n; i++)
	{
		for (j=0; j<n; j++)
		{
			cout << A[i][j] << "\t";
		}
		//   if (n>0), .. n > i >= 0  =>  n > 0
		cout << "| " << B[i] << "\n";
	}
}

int Gauss (double **A, double *B, double *X, int n)
{
	int i, j, k;
	//  
	cout << "Output A | B:\n";
	Output_Matrix_Vector (A, B, n);
	for (k=0; k<n-1; k++)
	{
		for (i=k+1; i<n; i++)
		{
			for (j=k+1; j<n; j++)
			{
				A[i][j] = A[i][j]*A[k][k]- A[k][j]*A[i][k];
			}
			B[i] = B[i]*A[k][k]- B[k]*A[i][k];
			A[i][k] = 0;
		}
	}
	cout << "Output A | B:\n";
	Output_Matrix_Vector (A, B, n);
	//  
	for (i=n-1; i>=0; i--)
	{
		if (fabs(A[i][i]) < EPS) return i+1; // i+1 > 0 - 
		X[i] = B[i];
		for (j=i+1; j<n; j++)
		{
			X[i] -= A[i][j]*X[j]; // a -= b; <=> a = a - b;
		}
		X[i] /= A[i][i];
	}
	return 0; //  
}

double** New_Array_2D (int ni, int nj)
{
	int i;
	double **X = new double* [ni]; //      (double*)
	for (i=0; i<ni; i++) X[i] = new double [nj]; //  i-   X    nj .
	return X;
}

void Delete_Array_2D (double **X, int n)
{
	int i;
	for (i=0; i<n; i++) delete[] X[i]; //   i- 
	delete[] X; //   
}

int main (void)
{
	cout << "Gauss\nversion 1.0\n"
		<<  "  a[0][0]*x[0] + ... +   a[0][j]*x[j] + ... +   a[0][n-1]*x[n-1] = b[0]\n"
		<<  "\t\t\t. . .\n"
		<<  "  a[i][0]*x[0] + ... +   a[i][j]*x[j] + ... +   a[i][n-1]*x[n-1] = b[i]\n"
		<<  "\t\t\t. . .\n"
		<<  "a[n-1][0]*x[0] + ... + a[n-1][j]*x[j] + ... + a[n-1][n-1]*x[n-1] = b[n-1]\n";
	cout << "Rang (n) = ";
	int n = 0;
	cin >> n;
	if (n<=0) return 1;

	cout << "A * X = B\n";
	cout << "Input matrix A:\n";
	double **A = New_Array_2D (n, n); // New
	Input_Matrix (A, n);
	cout << "Output matrix A:\n";
	Output_Matrix (A, n);
	cout << "Input vector B:\n";
	double *B = new double [n];
	Input_Vector (B, n);

	double *X = new double [n];
	int er = Gauss (A, B, X, n);
	if (er > 0) cout << "Error in " << er << " string!\n";
	else 
	{
		cout << "Vector X:\n";
		Output_Vector (X, n);
	}
	getch (); // .   Linux.  #include <conio.h> 
	delete[] X;
	delete[] B;
	Delete_Array_2D (A, n);
	return 0;
}